#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['bamboos', 'bamboos.utils', 'bamboos.utils.model']

package_data = \
{'': ['*']}

install_requires = \
['numpy',
 'pandas',
 'scikit-learn',
 'matplotlib',
 'seaborn',
 'lightgbm',
 'seaborn',
 'category_encoders>=2.0.0',
 'matplotlib-venn']

setup(name='bamboos',
      version='0.1.5',
      description='Wrapper Functions for pandas, numpy, and scikit learn',
      author='Aditya Kelvianto Sidharta',
      author_email='aditya.sdrt@gmail.com',
      url='http://github.com/AdityaSidharta/bamboos',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
     )
